subroutine modelsolution

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!!  Kazushige MATSUDA
  !!!!  "Progressive Taxation vs College Subsidies with College Dropout"
  !!!!
  !!!!
  !!!!  Routine to solve for equilibrium
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  use globals
  use nrtype
  use nr, only : polint, locate
  use nrutil, only : iminloc
  use interface, only : normcdf, valuev, linspace, util

  implicit none

  !local variables
  integer :: ITT, i, iii,fo=11,aatilde,aa2,hp2
  real(DP) :: distV0out, V00new(anum,hnum,qnum), Vaux0(anum,hnum,qnum),distpp0, Hnew(snum), LL_Nnew, LLnew, ybarnew, K_LLnew,taulnew,H_LL1new
  real(DP),save :: V00out(anum,hnum,qnum)=0.0d0,V00outnew(anum,hnum,qnum)=0.0d0,distV0
  real(DP),save :: Knew, Ynew, dnew

  if (wel==1 .or. wel==5) then
    grants(:,:,:)=grantprop*grantsopt(:,:,:)!*Eeq
  endif


  LL=LL_N*NN!labor
  K=K_LL*LL!capital

  Y=K**aalpha*LL**(1.0d0-aalpha)!GDP

  H(1) = H_LL1*LL
  H(3) = (max(1.0d0-a1*(H(1)/LL)**rho,0.001d0)/(1.0d0-a1))**(1.0d0/rho)*LL

  !!! Prices
  w(1)=a1*(1.0d0-aalpha)*(K/LL)**aalpha*(H(1)/LL)**(rho-1.0d0)
  w(3)=(1.0d0-a1)*(1.0d0-aalpha)*(K/LL)**aalpha*(H(3)/LL)**(rho-1.0d0)
  w(2)=cchiw*w(1)+(1.0d0-cchiw)*w(3) !wage for CD

  r = max(aalpha*(K_LL)**(aalpha-1.0d0)-ddelta,0.0d0)!saving interest rate
  rminus=r+iota!borrowing interest rate
  rs=rminus+iotas!student loan interest rate
  RR = 1.0d0+(1.0d0-tauk)*r!gross interest rate after tax
  RRdebt = 1.0d0+rminus!gross borrowing interest rate (no tax for borrowing)
  RRdebts= 1.0d0+rs!gross stundet loan interest
  tuition=w(1)*kappa! tuition

  !atilde(a): refinancing of student loan to a loan with rminus
  factor=rs/(1.0d0-(1.0d0+rs)**(-20/dj))*(1.0d0-(1.0d0+rminus)**(-20/dj))/rminus


  if (debug==1) then
    write(*,*) 'test', factor
    write(*,*) 'r,w=', r,w
    write(*,*) 'Y', Y,(r+ddelta)*K+w(1)*H(1)+w(3)*H(3)
    write(*,*) 'tuitions', tuition
  endif

  Aj(:,:) = 0.0d0

  if (calibflag==1) then

    Aj(1,ja+1) = bbase1/ybase*ybar/dble(dj)
    Aj(2,ja+1) = bbase1/ybase*ybar/dble(dj)
    Aj(1,ja+2) = bbase2/ybase*ybar/dble(dj)

    q1prime = q1base/ybase*ybar/(Y/NN)
    q2prime = q2base/ybase*ybar/(Y/NN)

    q1=q1base/ybase*ybar
    q2=q2base/ybase*ybar

    Z = d*Y/NN!d*ybar


    Aj(1,ja+3:jr-1)=blbase(1)/ybase*(ybar)/dble(dj)
    Aj(2,ja+2:jr-1)=blbase(2)/ybase*(ybar)/dble(dj)
    Aj(3,ja+1:jr-1)=blbase(3)/ybase*(ybar)/dble(dj)
    do qq=1,qnum
      do hh=1,hnum2
        grants(ja,hh,qq)=ttheta(ja,hh,qq)*ybar
        grants(ja+1,hh,qq)=ttheta(ja+1,hh,qq)*ybar
      enddo
    enddo

    if (debug==1) then
      write(*,*) 'ybar', 'Y', ybar, Y/NN, ybar/Y*NN
      write(*,*) 'gdp $', Y/ybar*ybase!/sum(Nvec(1:jend))
    endif

  else

    Aj(1,ja+1) = bbase1/ybase*YNeq/dble(dj)
    Aj(2,ja+1) = bbase1/ybase*YNeq/dble(dj)
    Aj(1,ja+2) = bbase2/ybase*YNeq/dble(dj)

    q1=q1primeeq*(Y/NN)
    q2=q2primeeq*(Y/NN)

    Z = d*Y/NN!d*YNeq
    Aj(1,ja+3:jr-1)=blbase(1)/ybase*YNeq/dble(dj)!*(Y/NN)
    Aj(2,ja+2:jr-1)=blbase(2)/ybase*YNeq/dble(dj)!*(Y/NN)
    Aj(3,ja+1:jr-1)=blbase(3)/ybase*YNeq/dble(dj)!*(Y/NN)

    if (debug==1) then
      write(*,*) 'gdp $', Y/YNeq*ybase!/sum(Nvec(1:jend))
    endif

  endif

  if (debug==1) then
    write(*,*) 'blimits', Aj(1,jr-1), Aj(2,jr-1), Aj(3,jr-1), Aj(2,ja+1), Aj(1,ja+2)
    write(*,*) 'grants', grants(ja,13,:)
    write(*,*) 'grants', grants(ja+1,13,:)
  endif

  Iamin=-transpose(Aj)

  Iamax(:,:)=6.25d0!+Iamin(:,:)
  Iamax(jend,:)=6.0d0!+Iamin(jend,:)
  Iamax(jb+2,:)=30.0d0!+Iamin(jb+1,:)
  Iamax(jb+1,:)=30.0d0!+Iamin(jb+1,:)
  Iamax(jb,:)=30.0d0!2.5d0!+Iamin(jb,:)
  Iamax(jr,:)=40.0d0!+Iamin(jr,:1
  Iamax(ja,:)=14.0d0!+Iamin(ja,:)
  Iamax(ja+1,:)=11.0d0!1.5d0!+Iamin(ja+1,:)
  Iamax(ja+2,:)=8.0d0!1.5d0!+Iamin(ja+1,:)

  Iamax(:,:)=Iamax(:,:)/12.2257895409d0

  do jj=ja+3,jb-1
    do s=1,snum
      Iamax(jj,s) = Iamax(ja+2,s) + dble(jj-ja-2)*(Iamax(jb,s)-Iamax(ja+2,s))/dble(jb-1-ja-2)
    enddo
  enddo
  do jj=jb+3,jr
    do s=1,snum
      Iamax(jj,s) = Iamax(jb+2,s) + dble(jj-jf-ja-2)*(Iamax(jr,s)-Iamax(jb+2,s))/dble(jr-jf-ja-2)
    enddo
  enddo
  do jj=jr+1,jend
    do s=1,snum
      Iamax(jj,s) = Iamax(jr,s) + dble(jj-jr)*(Iamax(jend,s)-Iamax(jr,s))/dble(jend-jr)
    enddo
  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  do jj=ja,jend
    do s=1,snum
      do aa=1,anum
        agrid(jj,aa,s)= (real(aa-1)/real(anum-1))**atheta*Iamax(jj,s)
      enddo
      do aa=1,anum2
        agrid2(jj,aa,s)=(real(aa-1)/real(anum2-1))**atheta*Iamax(jj,s)
      enddo

      agrid(jj,:,s)=agrid(jj,:,s)*(Iamax(jj,s)-Iamin(jj,s))/Iamax(jj,s)+Iamin(jj,s)
      agrid2(jj,:,s)=agrid2(jj,:,s)*(Iamax(jj,s)-Iamin(jj,s))/Iamax(jj,s)+Iamin(jj,s)

    enddo
  enddo

  do s=1,snum
    do aa=1,anum
      agridjf(aa,s)= (real(aa-1)/real(anum-1))**atheta*Iamax(jb+1,s)
    enddo
    agridjf(:,s)=agridjf(:,s)*(Iamax(jb+1,s)-Iamin(jb+1,s))/Iamax(jb+1,s)+Iamin(jb+1,s)

  enddo


  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! pension system
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  if (calibflag==1) then
    ybar2=ybarbase/ybase*ybar
  else
    ybar2=ybarbase/ybase*YNeq
  endif

  do s=1,snum
    ytrinterp(s,:) = w(s)*sum(aap(s,jc:jr-1))/dble(jr-jc)*ghgridinterp(s,:)*hbar
  enddo
  do hh=1,hnum2
    do s=1,snum
      if (ytrinterp(s,hh)<b1*ybar2) then
        piainterp(s,hh) = s1*ytrinterp(s,hh)
      elseif (ytrinterp(s,hh)<b2*ybar2) then
        piainterp(s,hh) = s1*b1*ybar2+s2*(ytrinterp(s,hh)-b1*ybar2)
      elseif (ytrinterp(s,hh)<b3*ybar2) then
        piainterp(s,hh) = s1*b1*ybar2+s2*(b2-b1)*ybar2+s3*(ytrinterp(s,hh)-b2*ybar2)
      else
        piainterp(s,hh) = s1*b1*ybar2+s2*(b2-b1)*ybar2+s3*(b3-b2)*ybar2
      endif
    enddo
  enddo

  if (calibflag==1) then
    do s=1,snum
      pinterp(s,:)=piainterp(s,:)-Z
    enddo
  else
    do s=1,snum
      pinterp(s,:)=piainterp(s,:)-Zeq
    enddo
  endif

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Household problem
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  do jj=jend,jr,-1
    do s=snum,1,-1
      eetai=1
      call valuev
    enddo
  enddo
  do jj=jr-1,jb+1,-1
    do s=snum,1,-1
      do eetai=1,2
        call valuev
      enddo
    enddo
  enddo

  distV0out = 200.0d0
  V00out(:,:,:)=0.0d0

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Outer loop
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  do i=1,10
    if (distV0out < tol) then
      exit
    endif

    Vs(:,:,:,:,:,:)=0.0d0
    V00 = V00out


    do jj=jb,ja,-1
      do s=snum,1,-1
        do eetai=1,2
          if (s==2 .and. jj==ja) then
            do ke=1,kenum
              call valuev
            enddo
          elseif (s==1 .and. jj==ja+1) then
            ke=1
            call valuev
          else
            call valuev
          endif
        enddo
      enddo
    enddo

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! Inner loop
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


    distV0 = 2000.0d0
    do iii=1,30

      if (distV0 < tol) exit

      do jj=jb,ja,-1
        do s=snum,1,-1
          if (jj==jb) then
            do eetai=1,2
              do hh=1,hnum
                do aa=1,anum
                  do hc=1,hnum

                    ww(hh)=w(s)*eeta(s,eetai)*aap(s,jj)*ghgrid(s,hh)

                    call polint(agrid(jj+1,:,s),V(:,hh,jj+1,s,1),amatjf(aa,hh,hc,s,eetai),Vprime(1),dy)
                    call polint(agrid(jj+1,:,s),V(:,hh,jj+1,s,2),amatjf(aa,hh,hc,s,eetai),Vprime(2),dy)

                    if (ww(hh)*hbase < q1) then
                      qq3=1
                    elseif (ww(hh)*hbase < q2) then
                      qq3=2
                    else
                      qq3=3
                    endif

                    call polint(agrid(ja,:,2),V00(:,hc,qq3),bmatjf(aa,hh,hc,s,eetai),V00prime,dy)

                    Vaux(aa,hh,hc)= util(cmatjf(aa,hh,hc,s,eetai),lmatjf(aa,hh,hc,s,eetai)) &
                    + bbeta*varphi(jj)*sum(pii(s,eetai,:)*Vprime(:))+vv*V00prime

                  enddo

                  do hc=1,hnum2
                    call polint(hgrid(:),Vaux(aa,hh,:),hgrid2(hc),Vauxinterph(aa,hh,hc),dy)
                  enddo
                  V(aa,hh,jj,s,eetai) = sum(psiem(:,s,hhm(hh))*Vauxinterph(aa,hh,:))
                enddo
              enddo
            enddo
          elseif (jy(s)<=jj .and. jj<jb) then
            do eetai=1,2
              do hh=1,hnum
                do aa=1,anum
                  call polint(agrid(jj+1,:,s),V(:,hh,jj+1,s,1),amat(aa,hh,jj,s,eetai),Vprime(1),dy)
                  call polint(agrid(jj+1,:,s),V(:,hh,jj+1,s,2),amat(aa,hh,jj,s,eetai),Vprime(2),dy)
                  V(aa,hh,jj,s,eetai)= util(cmat(aa,hh,jj,s,eetai)/(1.0d0+ind_fb(jj)*zzeta*f),lmat(aa,hh,jj,s,eetai))&
                  + bbeta*varphi(jj)*sum(pii(s,eetai,:)*Vprime(:))
                enddo
              enddo
            enddo
          elseif (s==1 .and. jj==ja+1) then
            do eetai=1,2
              do qq=1,qnum
                do thetac=1,hnum
                  do aa=1,anum
                    ke=1
                    call polint(agrid(jj+1,:,s),V(:,thetac,jj+1,s,1),asmat(aa,thetac,qq,jj,eetai,ke),Vprime(1),dy)
                    call polint(agrid(jj+1,:,s),V(:,thetac,jj+1,s,2),asmat(aa,thetac,qq,jj,eetai,ke),Vprime(2),dy)
                    Vs(aa,thetac,qq,jj,eetai,ke)= util(csmat(aa,thetac,qq,jj,eetai,ke),lsmat(aa,thetac,qq,jj,eetai,ke))&
                    +bbeta*varphi(jj)*sum(pis(s,:)*Vprime(:))

                    do thetah=1,hnum

                      ! if (agrid(jj,aa,s) < 0.0d0) then
                      !   call polint(agrid(jj,:,s),V(:,thetah,jj,2,eetai),agrid(jj,aa,s)*factor,Vwork,dy)
                      ! else
                      Vwork=V(aa,thetah,jj,2,eetai)
                      ! endif
                      do ke2=1,kenum

                        Veduc2=Vs(aa,thetac,qq,jj,eetai,1)+lambdaopt(jj,hhm(thetac),ke2)


                        if (Vwork>Veduc2) then
                          Vp(aa,thetac,qq,jj,eetai,ke2,thetah)=Vwork!
                        else
                          Vp(aa,thetac,qq,jj,eetai,ke2,thetah)=Veduc2
                        endif

                      enddo
                    enddo
                  enddo
                enddo
              enddo
              do ke2=1,kenum
                do qq=1,qnum
                  do thetah=1,hnum
                    do aa=1,anum
                      do thetac=1,hnum2
                        call polint(hgrid(:),Vp(aa,:,qq,jj,eetai,ke2,thetah),hgrid2(thetac),Vsprimeh(thetac),dy)
                      enddo
                      EVp(aa,thetah,qq,jj,eetai,ke2)=sum(pepmatopt(hhm(thetah),:)*Vsprimeh(:))
                    enddo
                  enddo
                enddo
              enddo
            enddo
          elseif (s==2 .and. jj==ja) then
            do ke=1,kenum
              do eetai=1,2
                do qq=1,qnum
                  do thetah=1,hnum
                    do aa=1,anum

                      call polint(agrid(jj+1,:,s),EVp(:,thetah,qq,jj+1,1,ke),asmat(aa,thetah,qq,jj,eetai,ke),Vsprime(1),dy)
                      call polint(agrid(jj+1,:,s),EVp(:,thetah,qq,jj+1,2,ke),asmat(aa,thetah,qq,jj,eetai,ke),Vsprime(2),dy)

                      Vs(aa,thetah,qq,jj,eetai,ke) = util(csmat(aa,thetah,qq,jj,eetai,ke),lsmat(aa,thetah,qq,jj,eetai,ke))&
                      +bbeta*sum(pis(s,:)*Vsprime(:))

                    enddo
                  enddo
                enddo
              enddo
            enddo

            do qq=1,qnum
              do thetah=1,hnum
                do aa=1,anum
                  do ke=1,kenum
                    do eetai=1,2

                      Veduc1=Vs(aa,thetah,qq,jj,eetai,ke)+lambdaopt(jj,hhm(thetah),ke)

                      if (V(aa,thetah,ja,3,eetai)>Veduc1) then
                        V00news(aa,thetah,qq,eetai,ke)=V(aa,thetah,ja,3,eetai)
                      else
                        V00news(aa,thetah,qq,eetai,ke)=Veduc1
                      endif
                    enddo
                  enddo
                  V00new(aa,thetah,qq)=pis(3,1)*sum(pike(:)*V00news(aa,thetah,qq,1,:))+pis(3,2)*sum(pike(:)*V00news(aa,thetah,qq,2,:))
                enddo
              enddo
            enddo
          endif
        enddo
      enddo

      distV0 = abs(maxval(maxval(maxval(abs(V00new-V00)/(1.0d0+abs(V00)),1),1),1))
      V00 = V00new

      if (iii>=30 .and. distV0>0.1d0) then
        write(*,*) 'V00 maxit', distV0
      endif

    enddo

    V00outnew = V00
    distV0out = abs(maxval(maxval(maxval(abs(V00outnew-V00out)/(1.0d0+abs(V00out)),1),1),1))
    V00out = V00outnew

    if (debug==1) then
      write(*,*) 'distV0out=', distV0out
    endif

    if (i>=10 .and. distV0out>0.1d0) then
      write(*,*) 'error: distV0 does not converge to zero'
    endif

  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !! Interpolation
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  do jj=jend,jr,-1
    do s=snum,1,-1
      eetai=1

      if (jr<=jj .and. jj<jend) then

        do hh=1,hnum
          do aa=1,anum2
            call polint(agrid(jj,:,s),armat(:,hh,jj,s),agrid2(jj,aa,s),armatinterp0(aa,hh,jj,s),dy)
            call polint(agrid(jj,:,s),crmat(:,hh,jj,s),agrid2(jj,aa,s),crmatinterp0(aa,hh,jj,s),dy)
          enddo
        enddo

        do hh2=1,hnum2
          do aa2=1,anum2

            call polint(hgrid(:),armatinterp0(aa2,:,jj,s),hgrid2(hh2),armatinterp(aa2,hh2,jj,s),dy)
            call polint(hgrid(:),crmatinterp0(aa2,:,jj,s),hgrid2(hh2),crmatinterp(aa2,hh2,jj,s),dy)

            call polint(agrid2(jj+1,:,s),Vrinterp(:,hh2,jj+1,s),armatinterp(aa2,hh2,jj,s),Vrprime,dy)
            Vrinterp(aa2,hh2,jj,s) = util(crmatinterp(aa2,hh2,jj,s),1.0_dp)+bbeta*varphi(jj)*Vrprime

            IIr(aa2,:,hh2,jj,s) = 0.0d0

            if (armatinterp(aa2,hh2,jj,s)-agrid2(jj+1,1,s)<=0.0d0) then
              IIr(aa2,1,hh2,jj,s) = 1.0d0
            elseif (armatinterp(aa2,hh2,jj,s)-agrid2(jj+1,anum2,s)>=0.0d0) then
             IIr(aa2,anum2,hh2,jj,s) = 1.0d0
           else

            aleft=max(locate(agrid2(jj+1,:,s),armatinterp(aa2,hh2,jj,s)), 1)
            aright=aleft+1

            if (aright>anum2) then
              write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
            endif
            IIr(aa2,aleft,hh2,jj,s) = (agrid2(jj+1,aright,s)-armatinterp(aa2,hh2,jj,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
            IIr(aa2,aright,hh2,jj,s) = (armatinterp(aa2,hh2,jj,s)-agrid2(jj+1,aleft,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
          endif

        enddo
      enddo

    elseif (jj==jend) then

      do hh=1,hnum
        do aa=1,anum2
          call polint(agrid(jj,:,s),crmat(:,hh,jj,s),agrid2(jj,aa,s),crmatinterp0(aa,hh,jj,s),dy)
        enddo
      enddo
      do hh2=1,hnum2
        do aa2=1,anum2
          call polint(hgrid(:),crmatinterp0(aa2,:,jj,s),hgrid2(hh2),crmatinterp(aa2,hh2,jj,s),dy)
          Vrinterp(aa2,hh2,jj,s) = util(crmatinterp(aa2,hh2,jj,s),1.0d0)
        enddo
      enddo
    endif

  enddo
enddo


do jj=jr-1,jb+1,-1
  do s=snum,1,-1
    do eetai=1,2
      do hh=1,hnum
        do aa2=1,anum2

          call polint(agrid(jj,:,s),amat(:,hh,jj,s,eetai),agrid2(jj,aa2,s),amatinterp0(aa2,hh,jj,s,eetai),dy)
          call polint(agrid(jj,:,s),cmat(:,hh,jj,s,eetai),agrid2(jj,aa2,s),cmatinterp0(aa2,hh,jj,s,eetai),dy)
          call polint(agrid(jj,:,s),lmat(:,hh,jj,s,eetai),agrid2(jj,aa2,s),lmatinterp0(aa2,hh,jj,s,eetai),dy)

        enddo
      enddo

      do hh=1,hnum2
        do aa2=1,anum2


          call polint(hgrid(:),amatinterp0(aa2,:,jj,s,eetai),hgrid2(hh),amatinterp(aa2,hh,jj,s,eetai),dy)
          call polint(hgrid(:),cmatinterp0(aa2,:,jj,s,eetai),hgrid2(hh),cmatinterp(aa2,hh,jj,s,eetai),dy)
          call polint(hgrid(:),lmatinterp0(aa2,:,jj,s,eetai),hgrid2(hh),lmatinterp(aa2,hh,jj,s,eetai),dy)

          if (jj==jr-1) then
            call polint(agrid2(jj+1,:,s),Vrinterp(:,hh,jj+1,s),amatinterp(aa2,hh,jj,s,eetai),Vrprime,dy)
            Vinterp(aa2,hh,jj,s,eetai)= util(cmatinterp(aa2,hh,jj,s,eetai),lmatinterp(aa2,hh,jj,s,eetai))&
            + bbeta*varphi(jj)*Vrprime
          else
            call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,1),amatinterp(aa2,hh,jj,s,eetai),Vprime(1),dy)
            call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,2),amatinterp(aa2,hh,jj,s,eetai),Vprime(2),dy)

            Vinterp(aa2,hh,jj,s,eetai)= util(cmatinterp(aa2,hh,jj,s,eetai),lmatinterp(aa2,hh,jj,s,eetai))&
            + bbeta*varphi(jj)*sum(pii(s,eetai,:)*Vprime(:))
          endif

          hmatinterp(aa2,hh,jj,s,eetai) = max(1.0d0-lmatinterp(aa2,hh,jj,s,eetai),0.0d0)

          hsmatinterp(aa2,hh,jj,s,eetai) = eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)*hmatinterp(aa2,hh,jj,s,eetai)
          ymatinterp(aa2,hh,jj,s,eetai) = w(s)*eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)*hmatinterp(aa2,hh,jj,s,eetai)

          if (amatinterp(aa2,hh,jj,s,eetai)<0.0d0) then
            borrowindinterp(aa2,hh,jj,s,eetai) = 1.0d0
          else
            borrowindinterp(aa2,hh,jj,s,eetai) = 0.0d0
          endif

          IIw(aa2,:,hh,jj,s,eetai) = 0.0d0

          if (amatinterp(aa2,hh,jj,s,eetai)-agrid2(jj+1,1,s)<=0.0d0) then
            IIw(aa2,1,hh,jj,s,eetai) = 1.0d0
          elseif (amatinterp(aa2,hh,jj,s,eetai)-agrid2(jj+1,anum2,s)>=0.0d0) then
            IIw(aa2,anum2,hh,jj,s,eetai) = 1.0d0
          else
            aleft=max(locate(agrid2(jj+1,:,s),amatinterp(aa2,hh,jj,s,eetai)), 1)
            aright=aleft+1

            if (aright>anum2) then
              write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
            endif
            IIw(aa2,aleft,hh,jj,s,eetai) = (agrid2(jj+1,aright,s)-amatinterp(aa2,hh,jj,s,eetai))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
            IIw(aa2,aright,hh,jj,s,eetai) = (amatinterp(aa2,hh,jj,s,eetai)-agrid2(jj+1,aleft,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))

          endif
        enddo
      enddo
    enddo
  enddo
enddo


do jj=jb,ja,-1
  do s=snum,1,-1
    do eetai=1,2

      if (s==1 .and. jj==jy(s)-1) then
        ke=1
        do qq=1,qnum

          do hh=1,hnum
            do aa2=1,anum2
              call polint(agrid(jj,:,s),asmat(:,hh,qq,jj,eetai,ke),agrid2(jj,aa2,s),asmatinterp0(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(agrid(jj,:,s),csmat(:,hh,qq,jj,eetai,ke),agrid2(jj,aa2,s),csmatinterp0(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(agrid(jj,:,s),lsmat(:,hh,qq,jj,eetai,ke),agrid2(jj,aa2,s),lsmatinterp0(aa2,hh,qq,jj,eetai,ke),dy)
            enddo
          enddo

          do hh=1,hnum2
            do aa2=1,anum2
              call polint(hgrid(:),asmatinterp0(aa2,:,qq,jj,eetai,ke),hgrid2(hh),asmatinterp(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(hgrid(:),csmatinterp0(aa2,:,qq,jj,eetai,ke),hgrid2(hh),csmatinterp(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(hgrid(:),lsmatinterp0(aa2,:,qq,jj,eetai,ke),hgrid2(hh),lsmatinterp(aa2,hh,qq,jj,eetai,ke),dy)

              call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,1),asmatinterp(aa2,hh,qq,jj,eetai,ke),Vprime(1),dy)
              call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,2),asmatinterp(aa2,hh,qq,jj,eetai,ke),Vprime(2),dy)

              Vsinterp(aa2,hh,qq,jj,eetai,ke)= util(csmatinterp(aa2,hh,qq,jj,eetai,ke),lsmatinterp(aa2,hh,qq,jj,eetai,ke))&
              + bbeta*sum(pis(s,:)*Vprime(:))

              hmatsinterp(aa2,hh,qq,jj,eetai,ke) = max(1.0d0-lsmatinterp(aa2,hh,qq,jj,eetai,ke)-stime,0.0d0)

              hssmatinterp(aa2,hh,qq,jj,eetai,ke) = eeta(s+1,eetai)*aap(s+1,jj)*ghgridinterp(s+1,hh)*hmatsinterp(aa2,hh,qq,jj,eetai,ke)
              ysmatinterp(aa2,hh,qq,jj,eetai,ke) = w(s+1)*eeta(s+1,eetai)*aap(s+1,jj)*ghgridinterp(s+1,hh)*hmatsinterp(aa2,hh,qq,jj,eetai,ke)


              if (asmatinterp(aa2,hh,qq,jj,eetai,ke)<0.0d0) then
                borrowindsinterp(aa2,hh,qq,jj,eetai,ke) = 1.0d0
              else
                borrowindsinterp(aa2,hh,qq,jj,eetai,ke) = 0.0d0
              endif

              IIws(aa2,:,hh,qq,s,eetai,ke) = 0.0d0

              if (asmatinterp(aa2,hh,qq,jj,eetai,ke)-agrid2(jj+1,1,s)<=0.0d0) then
                IIws(aa2,1,hh,qq,s,eetai,ke) = 1.0d0
              elseif (asmatinterp(aa2,hh,qq,jj,eetai,ke)-agrid2(jj+1,anum2,s)>=0.0d0) then
                IIws(aa2,anum2,hh,qq,s,eetai,ke) = 1.0d0
              else

                aleft=max(locate(agrid2(jj+1,:,s),asmatinterp(aa2,hh,qq,jj,eetai,ke)), 1)
                aright=aleft+1

                if (aright>anum2) then
                  write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
                endif
                IIws(aa2,aleft,hh,qq,s,eetai,ke) = (agrid2(jj+1,aright,s)-asmatinterp(aa2,hh,qq,jj,eetai,ke))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
                IIws(aa2,aright,hh,qq,s,eetai,ke) = (asmatinterp(aa2,hh,qq,jj,eetai,ke)-agrid2(jj+1,aleft,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
              endif

              do ke2=1,kenum

                Veduc2=Vsinterp(aa2,hh,qq,jj,eetai,1)+lambdaopt(jj,hh,ke2)

                asmatinterp(aa2,hh,qq,jj,eetai,ke2)=asmatinterp(aa2,hh,qq,jj,eetai,1)
                csmatinterp(aa2,hh,qq,jj,eetai,ke2)=csmatinterp(aa2,hh,qq,jj,eetai,1)
                lsmatinterp(aa2,hh,qq,jj,eetai,ke2)=lsmatinterp(aa2,hh,qq,jj,eetai,1)
                Vsinterp(aa2,hh,qq,jj,eetai,ke2)=Vsinterp(aa2,hh,qq,jj,eetai,1)
                hmatsinterp(aa2,hh,qq,jj,eetai,ke2)=hmatsinterp(aa2,hh,qq,jj,eetai,1)
                hssmatinterp(aa2,hh,qq,jj,eetai,ke2)=hssmatinterp(aa2,hh,qq,jj,eetai,1)
                ysmatinterp(aa2,hh,qq,jj,eetai,ke2)=ysmatinterp(aa2,hh,qq,jj,eetai,1)
                borrowindsinterp(aa2,hh,qq,jj,eetai,ke2)=borrowindsinterp(aa2,hh,qq,jj,eetai,1)
                IIws(aa2,:,hh,qq,s,eetai,ke2)=IIws(aa2,:,hh,qq,s,eetai,1)

                do thetah=1,hnum2

                  ! if (agrid2(jj,aa2,s) < 0.0d0) then
                  !   call polint(agrid2(jj,:,s),Vinterp(:,thetah,jj,2,eetai),agrid2(jj,aa2,s)*factor,Vwork,dy)
                  ! else
                  Vwork=Vinterp(aa2,thetah,jj,2,eetai)
                  ! endif

                  if (Vwork>Veduc2) then
                    Ipinterp(aa2,hh,qq,jj,eetai,ke2,thetah) = 0.0d0
                    Vpinterp(aa2,hh,qq,jj,eetai,ke2,thetah) = Vwork
                  else
                    Ipinterp(aa2,hh,qq,jj,eetai,ke2,thetah) = 1.0d0
                    Vpinterp(aa2,hh,qq,jj,eetai,ke2,thetah) = Veduc2
                  endif

                enddo
              enddo

            enddo
          enddo
        enddo

        do ke2=1,kenum
          do qq=1,qnum
            do hh=1,hnum2
              do aa=1,anum2
                EVpinterp(aa,hh,qq,jj,eetai,ke2)=sum(pepmatopt(hh,:)*Vpinterp(aa,:,qq,jj,eetai,ke2,hh))
              enddo
            enddo
          enddo
        enddo

      elseif (s==2 .and. jj==jy(s)-1) then

        do ke=1,kenum
          do qq=1,qnum
            do hh=1,hnum
             do aa2=1,anum2
              call polint(agrid(jj,:,s),asmat(:,hh,qq,jj,eetai,ke),agrid2(jj,aa2,s),asmatinterp0(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(agrid(jj,:,s),csmat(:,hh,qq,jj,eetai,ke),agrid2(jj,aa2,s),csmatinterp0(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(agrid(jj,:,s),lsmat(:,hh,qq,jj,eetai,ke),agrid2(jj,aa2,s),lsmatinterp0(aa2,hh,qq,jj,eetai,ke),dy)
            enddo
          enddo

          do hh=1,hnum2
            do aa2=1,anum2
              call polint(hgrid(:),asmatinterp0(aa2,:,qq,jj,eetai,ke),hgrid2(hh),asmatinterp(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(hgrid(:),csmatinterp0(aa2,:,qq,jj,eetai,ke),hgrid2(hh),csmatinterp(aa2,hh,qq,jj,eetai,ke),dy)
              call polint(hgrid(:),lsmatinterp0(aa2,:,qq,jj,eetai,ke),hgrid2(hh),lsmatinterp(aa2,hh,qq,jj,eetai,ke),dy)

              call polint(agrid2(jj+1,:,s),EVpinterp(:,hh,qq,jj+1,1,ke),asmatinterp(aa2,hh,qq,jj,eetai,ke),Vsprime(1),dy)
              call polint(agrid2(jj+1,:,s),EVpinterp(:,hh,qq,jj+1,2,ke),asmatinterp(aa2,hh,qq,jj,eetai,ke),Vsprime(2),dy)

              Vsinterp(aa2,hh,qq,jj,eetai,ke) = util(csmatinterp(aa2,hh,qq,jj,eetai,ke),lsmatinterp(aa2,hh,qq,jj,eetai,ke)) +bbeta*sum(pis(s,:)*Vsprime(:))

              hmatsinterp(aa2,hh,qq,jj,eetai,ke) = max(1.0d0-lsmatinterp(aa2,hh,qq,jj,eetai,ke)-stime,0.0d0)
              hssmatinterp(aa2,hh,qq,jj,eetai,ke) = eeta(s+1,eetai)*aap(s+1,jj)*ghgridinterp(s+1,hh)*hmatsinterp(aa2,hh,qq,jj,eetai,ke)
              ysmatinterp(aa2,hh,qq,jj,eetai,ke) = w(s+1)*eeta(s+1,eetai)*aap(s+1,jj)*ghgridinterp(s+1,hh)*hmatsinterp(aa2,hh,qq,jj,eetai,ke)

              if (asmatinterp(aa2,hh,qq,jj,eetai,ke)<0.0d0) then
                borrowindsinterp(aa2,hh,qq,jj,eetai,ke) = 1.0d0
              else
                borrowindsinterp(aa2,hh,qq,jj,eetai,ke) = 0.0d0
              endif

              IIws(aa2,:,hh,qq,s,eetai,ke) = 0.0d0

              if (asmatinterp(aa2,hh,qq,jj,eetai,ke)-agrid2(jj+1,1,s)<=0.0d0) then
                IIws(aa2,1,hh,qq,s,eetai,ke) = 1.0d0

              elseif (asmatinterp(aa2,hh,qq,jj,eetai,ke)-agrid2(jj+1,anum2,s)>=0.0d0) then
                IIws(aa2,anum2,hh,qq,s,eetai,ke) = 1.0d0
              else

                aleft=max(locate(agrid2(jj+1,:,s),asmatinterp(aa2,hh,qq,jj,eetai,ke)), 1)
                aright=aleft+1

                if (aright>anum2) then
                  write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
                endif
                IIws(aa2,aleft,hh,qq,s,eetai,ke) = (agrid2(jj+1,aright,s)-asmatinterp(aa2,hh,qq,jj,eetai,ke))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
                IIws(aa2,aright,hh,qq,s,eetai,ke) = (asmatinterp(aa2,hh,qq,jj,eetai,ke)-agrid2(jj+1,aleft,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
              endif


            enddo
          enddo
        enddo
      enddo

      do ke=1,kenum
        do qq=1,qnum
          do hh=1,hnum2
            do aa2=1,anum2

              Veduc1=Vsinterp(aa2,hh,qq,jj,eetai,ke)+lambdaopt(jj,hh,ke)

              if (Vinterp(aa2,hh,ja,3,eetai)>Veduc1) then
                I12pinterp(aa2,hh,qq,eetai,ke)=0.0d0
                V00interp(aa2,hh,qq,eetai,ke)=Vinterp(aa2,hh,ja,3,eetai)
              else
                I12pinterp(aa2,hh,qq,eetai,ke)=1.0d0
                V00interp(aa2,hh,qq,eetai,ke)=Veduc1
              endif

            enddo
          enddo
        enddo
      enddo

    elseif (jy(s)<=jj .and. jj<=jb) then
      if (jj == jb) then
        ! ww(:)=w(s)*eeta(s,eetai)*aap(s,jj)*ghgrid(s,:)
        do hc=1,hnum
          do hh=1,hnum
            do aa2=1,anum2
              call polint(agrid(jj,:,s),amatjf(:,hh,hc,s,eetai),agrid2(jj,aa2,s),amatjfinterp0(aa2,hh,hc,s,eetai),dy)
              call polint(agrid(jj,:,s),bmatjf(:,hh,hc,s,eetai),agrid2(jj,aa2,s),bmatjfinterp0(aa2,hh,hc,s,eetai),dy)
              call polint(agrid(jj,:,s),cmatjf(:,hh,hc,s,eetai),agrid2(jj,aa2,s),cmatjfinterp0(aa2,hh,hc,s,eetai),dy)
              call polint(agrid(jj,:,s),lmatjf(:,hh,hc,s,eetai),agrid2(jj,aa2,s),lmatjfinterp0(aa2,hh,hc,s,eetai),dy)
            enddo
          enddo
        enddo

        do hc=1,hnum
          do hh=1,hnum2
            do aa2=1,anum2
              call polint(hgrid(:),amatjfinterp0(aa2,:,hc,s,eetai),hgrid2(hh),amatjfinterp1(aa2,hh,hc,s,eetai),dy)
              call polint(hgrid(:),bmatjfinterp0(aa2,:,hc,s,eetai),hgrid2(hh),bmatjfinterp1(aa2,hh,hc,s,eetai),dy)
              call polint(hgrid(:),cmatjfinterp0(aa2,:,hc,s,eetai),hgrid2(hh),cmatjfinterp1(aa2,hh,hc,s,eetai),dy)
              call polint(hgrid(:),lmatjfinterp0(aa2,:,hc,s,eetai),hgrid2(hh),lmatjfinterp1(aa2,hh,hc,s,eetai),dy)
            enddo
          enddo
        enddo

        do hc=1,hnum2
          do hh=1,hnum2

            do aa2=1,anum2

              call polint(hgrid(:),amatjfinterp1(aa2,hh,:,s,eetai),hgrid2(hc),amatjfinterp(aa2,hh,hc,s,eetai),dy)
              call polint(hgrid(:),bmatjfinterp1(aa2,hh,:,s,eetai),hgrid2(hc),bmatjfinterp(aa2,hh,hc,s,eetai),dy)
              call polint(hgrid(:),cmatjfinterp1(aa2,hh,:,s,eetai),hgrid2(hc),cmatjfinterp(aa2,hh,hc,s,eetai),dy)
              call polint(hgrid(:),lmatjfinterp1(aa2,hh,:,s,eetai),hgrid2(hc),lmatjfinterp(aa2,hh,hc,s,eetai),dy)

              call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,1),amatjfinterp(aa2,hh,hc,s,eetai),Vprime(1),dy)
              call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,2),amatjfinterp(aa2,hh,hc,s,eetai),Vprime(2),dy)

              wwinterp(hh)=w(s)*eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)

              if (wwinterp(hh)*hbase < q1) then
                qq3=1
              elseif (wwinterp(hh)*hbase < q2) then
                qq3=2
              else
                qq3=3
              endif

              call polint(agrid(ja,:,2),V00(:,hc,qq3),bmatjfinterp(aa2,hh,hc,s,eetai),V00prime,dy)


              Vauxinterp(aa2,hh,hc)= util(cmatjfinterp(aa2,hh,hc,s,eetai),lmatjfinterp(aa2,hh,hc,s,eetai))&
              + bbeta*varphi(jj)*sum(pii(s,eetai,:)*Vprime(:))+vv*V00prime

              hmatjfinterp(aa2,hh,hc,s,eetai) = max(1.0d0-lmatjfinterp(aa2,hh,hc,s,eetai),0.0d0)

              hsmatjfinterp(aa2,hh,hc,s,eetai) = eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)*hmatjfinterp(aa2,hh,hc,s,eetai)
              ymatjfinterp(aa2,hh,hc,s,eetai) = w(s)*eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)*hmatjfinterp(aa2,hh,hc,s,eetai)

              if (amatjfinterp(aa2,hh,hc,s,eetai)<0.0d0) then
                borrowindjfinterp(aa2,hh,hc,s,eetai) = 1.0d0
              else
                borrowindjfinterp(aa2,hh,hc,s,eetai) = 0.0d0
              endif

              IIwjf(aa2,:,hh,hc,s,eetai) = 0.0d0

              if (amatjfinterp(aa2,hh,hc,s,eetai)-agrid2(jj+1,1,s)<=0.0d0) then
                IIwjf(aa2,1,hh,hc,s,eetai) = 1.0d0
              elseif (amatjfinterp(aa2,hh,hc,s,eetai)-agrid2(jj+1,anum2,s)>=0.0d0) then
                IIwjf(aa2,anum2,hh,hc,s,eetai) = 1.0d0
              else

                aleft=max(locate(agrid2(jj+1,:,s),amatjfinterp(aa2,hh,hc,s,eetai)), 1)
                aright=aleft+1

                if (aright>anum2) then
                  write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
                endif
                IIwjf(aa2,aleft,hh,hc,s,eetai) = (agrid2(jj+1,aright,s)-amatjfinterp(aa2,hh,hc,s,eetai))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
                IIwjf(aa2,aright,hh,hc,s,eetai) = (amatjfinterp(aa2,hh,hc,s,eetai)-agrid2(jj+1,aleft,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
              endif

              Ibwjf(aa2,:,hh,hc,s,eetai) = 0.0d0

              if (bmatjfinterp(aa2,hh,hc,s,eetai)-agrid2(ja,1,2)<=0.0d0) then
                Ibwjf(aa2,1,hh,hc,s,eetai) = 1.0d0

              elseif (bmatjfinterp(aa2,hh,hc,s,eetai)-agrid2(ja,anum2,2)>=0.0d0) then
                Ibwjf(aa2,anum2,hh,hc,s,eetai) = 1.0d0
              else

                aleft=max(locate(agrid2(ja,:,2),bmatjfinterp(aa2,hh,hc,s,eetai)), 1)
                aright=aleft+1

                if (aright>anum2) then
                  write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
                endif
                Ibwjf(aa2,aleft,hh,hc,s,eetai) = (agrid2(ja,aright,2)-bmatjfinterp(aa2,hh,hc,s,eetai))/(agrid2(ja,aright,2)-agrid2(ja,aleft,2))
                Ibwjf(aa2,aright,hh,hc,s,eetai) = (bmatjfinterp(aa2,hh,hc,s,eetai)-agrid2(ja,aleft,2))/(agrid2(ja,aright,2)-agrid2(ja,aleft,2))

              endif
            enddo
          enddo
        enddo


        do hh=1,hnum2
          do aa=1,anum2
            Vinterp(aa,hh,jj,s,eetai) = sum(psiem(:,s,hh)*Vauxinterp(aa,hh,:))
            cmatinterp(aa,hh,jj,s,eetai) = sum(cmatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
            hmatinterp(aa,hh,jj,s,eetai) = sum(hmatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
            lmatinterp(aa,hh,jj,s,eetai)=1.0d0-hmatinterp(aa,hh,jj,s,eetai)
            hsmatinterp(aa,hh,jj,s,eetai) = sum(hsmatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
            ymatinterp(aa,hh,jj,s,eetai) = sum(ymatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
            amatinterp(aa,hh,jj,s,eetai) = sum(amatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
            bmatinterp(aa,hh,jj,s,eetai) = sum(bmatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
            borrowindinterp(aa,hh,jj,s,eetai) = sum(borrowindjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
          enddo
        enddo

      else

        do hh=1,hnum
          do aa2=1,anum2
            call polint(agrid(jj,:,s),amat(:,hh,jj,s,eetai),agrid2(jj,aa2,s),amatinterp0(aa2,hh,jj,s,eetai),dy)
            call polint(agrid(jj,:,s),cmat(:,hh,jj,s,eetai),agrid2(jj,aa2,s),cmatinterp0(aa2,hh,jj,s,eetai),dy)
            call polint(agrid(jj,:,s),lmat(:,hh,jj,s,eetai),agrid2(jj,aa2,s),lmatinterp0(aa2,hh,jj,s,eetai),dy)
          enddo
        enddo

        do hh=1,hnum2
          do aa2=1,anum2
            call polint(hgrid(:),amatinterp0(aa2,:,jj,s,eetai),hgrid2(hh),amatinterp(aa2,hh,jj,s,eetai),dy)
            call polint(hgrid(:),cmatinterp0(aa2,:,jj,s,eetai),hgrid2(hh),cmatinterp(aa2,hh,jj,s,eetai),dy)
            call polint(hgrid(:),lmatinterp0(aa2,:,jj,s,eetai),hgrid2(hh),lmatinterp(aa2,hh,jj,s,eetai),dy)

            call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,1),amatinterp(aa2,hh,jj,s,eetai),Vprime(1),dy)
            call polint(agrid2(jj+1,:,s),Vinterp(:,hh,jj+1,s,2),amatinterp(aa2,hh,jj,s,eetai),Vprime(2),dy)
            Vinterp(aa2,hh,jj,s,eetai)= util(cmatinterp(aa2,hh,jj,s,eetai)/(1.0d0+ind_fb(jj)*zzeta*f),lmatinterp(aa2,hh,jj,s,eetai))&
            + bbeta*varphi(jj)*sum(pii(s,eetai,:)*Vprime(:))

            hmatinterp(aa2,hh,jj,s,eetai) = max(1.0d0-lmatinterp(aa2,hh,jj,s,eetai),0.0d0)
            hsmatinterp(aa2,hh,jj,s,eetai) = eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)*hmatinterp(aa2,hh,jj,s,eetai)
            ymatinterp(aa2,hh,jj,s,eetai) = w(s)*eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)*hmatinterp(aa2,hh,jj,s,eetai)

            if (amatinterp(aa2,hh,jj,s,eetai)<0.0d0) then
              borrowindinterp(aa2,hh,jj,s,eetai) = 1.0d0
            else
              borrowindinterp(aa2,hh,jj,s,eetai) = 0.0d0
            endif

            IIw(aa2,:,hh,jj,s,eetai) = 0.0d0

            if (amatinterp(aa2,hh,jj,s,eetai)-agrid2(jj+1,1,s)<=0.0d0) then
              IIw(aa2,1,hh,jj,s,eetai) = 1.0d0
            elseif (amatinterp(aa2,hh,jj,s,eetai)-agrid2(jj+1,anum2,s)>=0.0d0) then
              IIw(aa2,anum2,hh,jj,s,eetai) = 1.0d0
            else

              aleft=max(locate(agrid2(jj+1,:,s),amatinterp(aa2,hh,jj,s,eetai)), 1)
              aright=aleft+1

              if (aright>anum2) then
                write(*,*),'aright over',jj,s,eetai,aleft,aright,anum2
              endif
              IIw(aa2,aleft,hh,jj,s,eetai) = (agrid2(jj+1,aright,s)-amatinterp(aa2,hh,jj,s,eetai))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
              IIw(aa2,aright,hh,jj,s,eetai) = (amatinterp(aa2,hh,jj,s,eetai)-agrid2(jj+1,aleft,s))/(agrid2(jj+1,aright,s)-agrid2(jj+1,aleft,s))
            endif
          enddo
        enddo

      endif

    endif
  enddo
enddo
enddo


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Finding mass
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
pp0(:,:,:)=1.0d0
pp0 = pp0/sum(pp0)
distpp0 = 100.0d0

ppws(:,:,:,:,:,:,:)=0.0d0
ppws0(:,:,:,:,:,:,:)=0.0d0
ppw(:,:,:,:,:)=0.0d0
ppr(:,:,:,:)=0.0d0
ppbjf(:,:,:,:,:,:)=0.0d0

pp0 = pp0/sum(pp0)

do i=1,maxit
  if (distpp0 < tol/100d0) then
    exit
  endif

  do ke=1,kenum
    do eetai=1,2
      do qq=1,qnum
        do thetah=1,hnum2
          do aa=1,anum2
            ppws(aa,thetah,qq,ja,2,eetai,ke) = pike(ke)*pis(3,eetai)*(I12pinterp(aa,thetah,qq,eetai,ke)*pp0(aa,thetah,qq))
            ppws(aa,thetah,qq,ja,3,eetai,ke) = pike(ke)*pis(3,eetai)*(1.0d0-I12pinterp(aa,thetah,qq,eetai,ke))*pp0(aa,thetah,qq)
          enddo
        enddo
      enddo
    enddo
  enddo

  ppw(:,:,ja,2,:)=sum(sum(ppws(:,:,:,ja,2,:,:),5),3)
  ppw(:,:,ja,3,:)=sum(sum(ppws(:,:,:,ja,3,:,:),5),3)

  do thetah=1,hnum2
    do ke=1,kenum
      do eetai=1,2
        do qq=1,qnum
          do thetac=1,hnum2
            do aa=1,anum2
              ppws0(aa,thetac,qq,ja+1,eetai,ke,thetah) = &
              pepmat(thetah,thetac)*sum(IIws(:,aa,thetah,qq,2,eetai,ke)*ppws(:,thetah,qq,ja,2,eetai,ke),1)
            enddo
          enddo
        enddo
      enddo
    enddo
  enddo

  ! jj=ja
  ! s=1
  do ke=1,kenum
    do eetai=1,2
      do qq=1,qnum
        do hh=1,hnum2
          do aa=1,anum2
            ppws(aa,hh,qq,ja+1,1,eetai,ke) = &
            sum(Ipinterp(aa,hh,qq,ja+1,eetai,ke,:)*pis(2,eetai)*(ppws0(aa,hh,qq,ja+1,1,ke,:)+ppws0(aa,hh,qq,ja+1,2,ke,:)))
          enddo
        enddo
      enddo
    enddo
  enddo

  do ke=1,kenum
    do eetai=1,2
      do qq=1,qnum
        do hh=1,hnum2
          do aa=1,anum2
            ppws(aa,hh,qq,ja+1,2,eetai,ke) = &
            sum((1.0d0-Ipinterp(aa,:,qq,ja+1,eetai,ke,hh))*pis(2,eetai)*(ppws0(aa,:,qq,ja+1,1,ke,hh)+ppws0(aa,:,qq,ja+1,2,ke,hh)))
          enddo
        enddo
      enddo
    enddo
  enddo


  ppw(:,:,ja+1,1,:)=sum(sum(ppws(:,:,:,ja+1,1,:,:),5),3)
  ppw(:,:,ja+1,2,:)=sum(sum(ppws(:,:,:,ja+1,2,:,:),5),3)

  s=3
  jj=ja
  do eetai=1,2
    do hh=1,hnum2
      do aa=1,anum2
        ppw(aa,hh,jj+1,s,eetai) = pii(s,1,eetai)*dot_product(IIw(:,aa,hh,jj,s,1),ppw(:,hh,jj,s,1))&
        + pii(s,2,eetai)*dot_product(IIw(:,aa,hh,jj,s,2),ppw(:,hh,jj,s,2))
      enddo
    enddo
  enddo

  s=1
  do ke=1,kenum
    do eetai=1,2
      do qq=1,qnum
        do hh=1,hnum2
          do aa=1,anum2
            sumppw(aa,hh,qq,eetai,ke)=pis(1,eetai)*dot_product(IIws(:,aa,hh,qq,1,1,ke),ppws(:,hh,qq,ja+1,1,1,ke))&
            +pis(1,eetai)*dot_product(IIws(:,aa,hh,qq,1,2,ke),ppws(:,hh,qq,ja+1,1,2,ke))
          enddo
        enddo
      enddo
    enddo
  enddo

  ppw(:,:,ja+2,1,:) = sum(sum(sumppw(:,:,:,:,:),5),3)

  s=2
  jj=ja+1
  do eetai=1,2
    do hh=1,hnum2
      do aa=1,anum2
        ppw(aa,hh,jj+1,s,eetai) = pii(s,1,eetai)*dot_product(IIw(:,aa,hh,jj,s,1),ppw(:,hh,jj,s,1))&
        + pii(s,2,eetai)*dot_product(IIw(:,aa,hh,jj,s,2),ppw(:,hh,jj,s,2))
      enddo
    enddo
  enddo

  s=3
  jj=ja+1
  do eetai=1,2
    do hh=1,hnum2
      do aa=1,anum2
        ppw(aa,hh,jj+1,s,eetai) = pii(s,1,eetai)*dot_product(IIw(:,aa,hh,jj,s,1),ppw(:,hh,jj,s,1))&
        + pii(s,2,eetai)*dot_product(IIw(:,aa,hh,jj,s,2),ppw(:,hh,jj,s,2))
      enddo
    enddo
  enddo

  do jj=ja+2,jb-1
    do eetai=1,2
      do s=1,snum
        do hh=1,hnum2
          do aa=1,anum2
            ppw(aa,hh,jj+1,s,eetai) = pii(s,1,eetai)*dot_product(IIw(:,aa,hh,jj,s,1),ppw(:,hh,jj,s,1))&
            + pii(s,2,eetai)*dot_product(IIw(:,aa,hh,jj,s,2),ppw(:,hh,jj,s,2))
          enddo
        enddo
      enddo
    enddo
  enddo

  jj=jb
  do eetai=1,2
    do s=1,snum
      do hc=1,hnum2
        do hh=1,hnum2
          do aa2=1,anum2
            mp= max(w(s)*eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh),m)
            if (mp*hbase < q1) then
              ppbjf(aa2,hh,hc,s,eetai,1)=sum(Ibwjf(:,aa2,hh,hc,s,eetai)*ppw(:,hh,jj,s,eetai))*psiem(hc,s,hh)
            elseif (mp*hbase < q2) then
              ppbjf(aa2,hh,hc,s,eetai,2)=sum(Ibwjf(:,aa2,hh,hc,s,eetai)*ppw(:,hh,jj,s,eetai))*psiem(hc,s,hh)
            else
              ppbjf(aa2,hh,hc,s,eetai,3)=sum(Ibwjf(:,aa2,hh,hc,s,eetai)*ppw(:,hh,jj,s,eetai))*psiem(hc,s,hh)
            endif
          enddo
        enddo
      enddo
    enddo
  enddo


  pp0new(:,:,:) = sum(sum(sum(ppbjf(:,:,:,:,:,:),5),4),2)
  distpp0 = maxval(maxval(maxval(abs(pp0new-pp0),1),1),1)
  pp0 = pp0new*revise10 + pp0*(1.0d0-revise10)

  if (iii>=maxit .and. distpp0>0.1d0) then
    write(*,*) 'pp0 maxit', distpp0
    stop
  endif

enddo


do ke=1,kenum
  do eetai=1,2
    do qq=1,qnum
      do hh=1,hnum2
        do aa=1,anum2
          pppp0(aa,hh,qq,eetai,ke)=pp0(aa,hh,qq)*pis(3,eetai)*pike(ke)
        enddo
      enddo
    enddo
  enddo
enddo


jj=jb

do eetai=1,2
  do s=1,snum
    do hc=1,hnum2
      do hh=1,hnum2
        do aa2=1,anum2
          ppwjf(aa2,hh,hc,s,eetai)=sum(IIwjf(:,aa2,hh,hc,s,eetai)*ppw(:,hh,jj,s,eetai))*psiem(hc,s,hh)
        enddo
      enddo
    enddo
  enddo
enddo

do eetai=1,2
  do s=1,snum
    do hh=1,hnum2
      do aa=1,anum2
        ppw(aa,hh,jj+1,s,eetai) = pii(s,1,eetai)*sum(ppwjf(aa,hh,:,s,1))+pii(s,2,eetai)*sum(ppwjf(aa,hh,:,s,2))
      enddo
    enddo
  enddo
enddo

do s=1,snum
  do jj=jb+1,jr-2
    do eetai=1,2
      do hh=1,hnum2
        do aa=1,anum2
          ppw(aa,hh,jj+1,s,eetai) = pii(s,1,eetai)*dot_product(IIw(:,aa,hh,jj,s,1),ppw(:,hh,jj,s,1))&
          + pii(s,2,eetai)*dot_product(IIw(:,aa,hh,jj,s,2),ppw(:,hh,jj,s,2))
        enddo
      enddo
    enddo
  enddo
enddo

do s=1,snum
  do hh=1,hnum2
    do aa=1,anum2
      ppr(aa,hh,jr,s) = (dot_product(IIw(:,aa,hh,jr-1,s,1),ppw(:,hh,jr-1,s,1))+ dot_product(IIw(:,aa,hh,jr-1,s,2),ppw(:,hh,jr-1,s,2)))
    enddo
  enddo
enddo

do s=1,snum
  do jj=jr,jend-1
    do hh=1,hnum2
      do aa=1,anum2
        ppr(aa,hh,jj+1,s) = dot_product(IIr(:,aa,hh,jj,s),ppr(:,hh,jj,s))
      enddo
    enddo
  enddo
enddo

if (debug==1) then

  write(*,*) 'pp0=', sum(pp0)
  write(*,*) 'ppr=', sum(ppr(:,:,jr,:))
  write(*,*) 'ppend=', sum(ppr(:,:,jend,:))
  write(*,*) 'ppw=', sum(ppw(:,:,jb,:,:))
endif

if (abs(1.0_dp-sum(pp0))>tol .or. abs(1.0_dp-sum(ppr(:,:,jend,:)))>tol) then
  stop 'error: population does not converge'
endif

ytpw(:,:,:,:,:)=0.0d0

do eetai=1,2

  do s=1,snum
    do jj=jy(s),jr-1
      do hh=1,hnum2
        do aa=1,anum2
          hpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*hmatinterp(aa,hh,jj,s,eetai)
          lpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*lmatinterp(aa,hh,jj,s,eetai)
          hspw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*hsmatinterp(aa,hh,jj,s,eetai)
          wwpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*w(s)*eeta(s,eetai)*aap(s,jj)*ghgridinterp(s,hh)
          cpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*cmatinterp(aa,hh,jj,s,eetai)
          apw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*amatinterp(aa,hh,jj,s,eetai)
          ypw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*ymatinterp(aa,hh,jj,s,eetai)
          borrowpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*borrowindinterp(aa,hh,jj,s,eetai)

          if (jj==jb) then
            ytpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*sum(ymatjfinterp(aa,hh,:,s,eetai)*psiem(:,s,hh))
          else
            ytpw(aa,hh,jj,s,eetai)=Nvec(jj)*ppw(aa,hh,jj,s,eetai)*ymatinterp(aa,hh,jj,s,eetai)
          endif
        enddo
      enddo
    enddo
  enddo
enddo

do s=1,2
  jj=jy(s)-1
  do hh=1,hnum2
    do aa=1,anum2
      do eetai=1,2
        hpw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*hmatsinterp(aa,hh,:,jj,eetai,:))
        lpw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*lsmatinterp(aa,hh,:,jj,eetai,:))
        hspw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*hssmatinterp(aa,hh,:,jj,eetai,:))
        wwpw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*w(s+1)*eeta(s+1,eetai)*aap(s+1,jj)*ghgridinterp(s+1,hh))
        cpw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*csmatinterp(aa,hh,:,jj,eetai,:))
        apw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*asmatinterp(aa,hh,:,jj,eetai,:))
        ypw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*ysmatinterp(aa,hh,:,jj,eetai,:))
        borrowpw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*borrowindsinterp(aa,hh,:,jj,eetai,:))
        borrowam(aa,hh,jj,s,eetai)=-Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*borrowindsinterp(aa,hh,:,jj,eetai,:)*asmatinterp(aa,hh,:,jj,eetai,:))
        ytpw(aa,hh,jj,s,eetai)=Nvec(jj)*sum(ppws(aa,hh,:,jj,s,eetai,:)*ysmatinterp(aa,hh,:,jj,eetai,:))
      enddo
    enddo
  enddo
enddo

do s=1,snum
  do jj=jr,jend
    do hh=1,hnum2
      do aa=1,anum2
        cpr(aa,hh,jj,s) = Nvec(jj)*ppr(aa,hh,jj,s)*crmatinterp(aa,hh,jj,s)
        lpr(aa,hh,jj,s) = Nvec(jj)*ppr(aa,hh,jj,s)*1.0d0
        apr(aa,hh,jj,s) = Nvec(jj)*ppr(aa,hh,jj,s)*armatinterp(aa,hh,jj,s)
      enddo
    enddo
  enddo
enddo
do eetai=1,2
  do s=1,snum
    do hh=1,hnum2
      do aa=1,anum2
        bpwjf(aa,hh,s,eetai) = Nvec(jb)*ppw(aa,hh,jb,s,eetai)*bmatinterp(aa,hh,jb,s,eetai)
      enddo
    enddo
  enddo
enddo

P1 = sum(ppw(:,:,ja+1,1,:))
P3 = sum(ppw(:,:,ja,3,:))
P2 = sum(ppw(:,:,ja+1,2,:))



do s=1,snum
  C(s) = sum(cpw(:,:,ja:jr-1,s,:)) + sum(cpr(:,:,jr:jend,s))
  Bt(s) = sum(bpwjf(:,:,s,:))
  wealth(s) = sum(apw(:,:,ja:jr-1,s,:)) + sum(apr(:,:,jr:jend,s))
enddo

borrowagg = sum(borrowpw(:,:,ja+2:jr-1,:,:))/sum(Nvec(ja+2:jr-1))

Hnew(1) = sum(hspw(:,:,ja+2:jr-1,1,:))-kappa*(P1*(Nvec(ja)+Nvec(ja+1))+P2*Nvec(ja))
Hnew(2) = sum(hspw(:,:,ja+1:jr-1,2,:))+sum(hspw(:,:,ja+1,1,:))
Hnew(3) = sum(hspw(:,:,:,3,:))+sum(hspw(:,:,ja,2,:))

Hnew(1) = max(Hnew(1)+cchiw*Hnew(2),m)
Hnew(3) = max(Hnew(3)+(1.0d0-cchiw)*Hnew(2),m)
Hnew(2) = 0.0d0!max(Hnew(2),m)

LLnew = (a1*Hnew(1)**rho+(1.0d0-a1)*Hnew(3)**rho)**(1.0d0/rho)
LL_Nnew = LLnew/NN
Knew = max(sum(wealth)/(1.0d0+cchi),m)
Ynew = max(Knew**aalpha*LLnew**(1.0d0-aalpha),m)
K_LLnew=Knew/LLnew

do s=1,snum
  do jj=jr,jend
    sumom(s,jj)= sum(sum(ppr(:,:,jj,s),1)*pinterp(s,:),1)*Nvec(jj)
  enddo
enddo

thetap(ja)=sum(sum(sum(sum(ppws(:,:,:,ja,2,:,:),5),4),1)*grants(ja,:,:))
thetap(ja+1)=sum(sum(sum(sum(ppws(:,:,:,ja+1,1,:,:),5),4),1)*grants(ja+1,:,:))

E = (Nvec(ja)*thetap(ja)+Nvec(ja+1)*thetap(ja+1))!&
!+r*sum(borrowam(:,:,ja,:,:))-sum(borrowam(:,:,ja+1,:,:))*(1.0d0/factor-1.0d0)

E0=Nvec(ja)*thetap(ja)+Nvec(ja+1)*thetap(ja+1)
GDP=Ynew+w(1)*kappa*(P1*(Nvec(ja)+Nvec(ja+1))+P2*Nvec(ja))

taulnew = min((gy*Ynew+E+sum(sumom)+Z*sum(Nvec(ja:jend))-tauc*sum(C)-tauk*r*Knew+Nvec(ja)*trans0)/sum(ytpw),0.99d0)

H_LL1new = Hnew(1)/LLnew
L(1) = sum(hpw(:,:,ja+2:jr-1,1,:))
L(2) = sum(hpw(:,:,ja+1:jr-1,2,:))+sum(hpw(:,:,ja+1,1,:))
L(3) = sum(hpw(:,:,ja:jr-1,3,:))+sum(hpw(:,:,ja,2,:))

if (peind==1) then
  write(*,*) 'P1=', P1
  write(*,*) 'P2=', P2
  write(*,*) 'P3=', P3

  write(*,*) 'enroll=', P1+P2
  write(*,*) 'grad=', P1
  write(*,*) 'grantsja', grants(ja,13,:)/YNeq*ybase
  write(*,*) 'grantsja+1', grants(ja+1,13,:)/YNeq*ybase
  write(*,*) 'enrollgrad', P1+P2,P1
  write(*,*) 'skill prem', mskillp1, mskillp2
  stop
endif

if (P3>0.001d0) then
  ybarnew=sum(ypw(:,:,jb,3,:))/Nvec(jb)/sum(ppw(:,:,jb,3,:))!P3
else
  ybarnew=2.34d0
endif

if (debug==1) then
  write(*,*) 'rho', 1.0d0/sqrt((stddc/stddh)**2+1.0d0)
  write(*,*) 'school cost', lambda(ja,hhm(3),15)/sum(pppp0(:,:,:,:,:)*V00interp(:,:,:,:,:))
  write(*,*) 'P1=', P1
  write(*,*) 'P2=', P2
  write(*,*) 'P3=', P3
  write(*,*) 'demand=',(cchi+ddelta)*Knew/Ynew+sum(C)/Ynew+gy!+kappa*w(1)*Nvec(ja)*P1/Ynew
  write(*,*) 'taul=',taul
endif

! write(*,*) 'avg income',ybarnew

if (taxelas==1) then

  if (delas==1) then

    dnew=(taul*sum(ytpw)+tauc*sum(C)+tauk*r*Knew-gy*Ynew-E-sum(sumom)-Nvec(ja)*trans0)/sum(Nvec(ja:jend))*NN/Ynew

    fval2(1) = K_LLnew-K_LL
    fval2(2) = H_LL1new-H_LL1
    fval2(3) = LL_Nnew-LL_N
    fval2(4) = dnew-d
    ! fval(5) = (grantpropnew-grantprop)/(1.0d0+abs(grantprop))

    if (debug==1) then
      write(*,*) 'moments',K_LL, H_LL1, LL_N, d
    endif

    dddddd=maxval(abs(fval2))

    K_LL = revise*K_LLnew + (1.0d0-revise)*K_LL
    K_LL = max(K_LL,m)
    H_LL1 = revise3*H_LL1new + (1.0d0-revise3)*H_LL1
    H_LL1 = max(H_LL1,m)
    LL_N = revise*LL_Nnew + (1.0d0-revise)*LL_N
    LL_N = max(LL_N,m)
    d = revise*dnew + (1.0d0-revise)*d
    grantprop=1.0d0!revise*grantpropnew + (1.0d0-revise)*grantprop

  else

    grantpropnew&
    =(taul*sum(ytpw)-gy*Ynew-sum(sumom)-Z*sum(Nvec(ja:jend))+tauc*sum(C)+tauk*r*Knew-Nvec(ja)*trans0)/(Nvec(ja)*sum(sum(sum(sum(ppws(:,:,:,ja,2,:,:),5),4),1)*grantsopt(ja,:,:))+Nvec(ja+1)*sum(sum(sum(sum(ppws(:,:,:,ja+1,1,:,:),5),4),1)*grantsopt(ja+1,:,:)))!/Eeq

    fval2(1) = K_LLnew-K_LL
    fval2(2) = H_LL1new-H_LL1
    fval2(3) = LL_Nnew-LL_N
    fval2(4) = grantpropnew-grantprop

    if (debug==1) then
      write(*,*) 'moments',K_LL, H_LL1, LL_N, grantprop
    endif

    dddddd=maxval(abs(fval2))

    K_LL = revise*K_LLnew + (1.0d0-revise)*K_LL
    K_LL = max(K_LL,m)
    H_LL1 = revise3*H_LL1new + (1.0d0-revise3)*H_LL1
    H_LL1 = max(H_LL1,m)
    LL_N = revise*LL_Nnew + (1.0d0-revise)*LL_N
    LL_N = max(LL_N,m)
    grantprop=revise*grantpropnew + (1.0d0-revise)*grantprop
  endif

else

  if (calibflag==1) then

    fval(1) = K_LLnew-K_LL
    fval(2) = H_LL1new-H_LL1
    fval(3) = LL_Nnew-LL_N
    fval(4) = taulnew-taul
    fval(5) = ybarnew-ybar

    K_LL = revise*K_LLnew + (1.0d0-revise)*K_LL
    K_LL = max(K_LL,m)
    if (nodropout==1) then
      H_LL1 = revise*H_LL1new + (1.0d0-revise)*H_LL1
    else
      if (heterostddc==1) then
        H_LL1 = revise10*H_LL1new + (1.0d0-revise10)*H_LL1
      else
        H_LL1 = revise3*H_LL1new + (1.0d0-revise3)*H_LL1
      endif
    endif
    H_LL1 = max(H_LL1,m)
    LL_N = revise*LL_Nnew + (1.0d0-revise)*LL_N
    LL_N = max(LL_N,m)
    taul = revise*taulnew + (1.0d0-revise)*taul
    ybar = revise*ybarnew+(1.0d0-revise)*ybar


    dddddd=maxval(abs(fval))

  else


    if (wel==1) then

      if (Nvec(ja)*sum(sum(sum(sum(ppws(:,:,:,ja,2,:,:),5),4),1)*grantsopt(ja,:,:))+Nvec(ja+1)*sum(sum(sum(sum(ppws(:,:,:,ja+1,1,:,:),5),4),1)*grantsopt(ja+1,:,:)) < m) then
        grantpropnew=grantprop-0.1d0
      else
        grantpropnew=Efix/(Nvec(ja)*sum(sum(sum(sum(ppws(:,:,:,ja,2,:,:),5),4),1)*grantsopt(ja,:,:))+Nvec(ja+1)*sum(sum(sum(sum(ppws(:,:,:,ja+1,1,:,:),5),4),1)*grantsopt(ja+1,:,:)))
      endif

      fval(1) = K_LLnew-K_LL
      fval(2) = H_LL1new-H_LL1
      fval(3) = LL_Nnew-LL_N
      fval(4) = taulnew-taul
      fval(5)=(grantpropnew-grantprop)/(1.0d0+abs(grantprop))


      if (debug==1) then
        write(*,*) 'moments',K_LL, H_LL1, LL_N, taul, grantprop
      endif

      dddddd=maxval(abs(fval))

      if(weltest==19)then
        K_LL = 0.1d0*K_LLnew + (1.0d0-0.1d0)*K_LL
        K_LL = max(K_LL,m)
        H_LL1 = 0.05d0*H_LL1new + (1.0d0-0.05d0)*H_LL1
      else
        K_LL = revise*K_LLnew + (1.0d0-revise)*K_LL
        K_LL = max(K_LL,m)
        if (heterostddc==1) then
          H_LL1 = revise10*H_LL1new + (1.0d0-revise10)*H_LL1
        else
          H_LL1 = revise3*H_LL1new + (1.0d0-revise3)*H_LL1
        endif


        ! H_LL1 = revise3*H_LL1new + (1.0d0-revise3)*H_LL1
      endif
      H_LL1 = max(H_LL1,m)
      LL_N = revise*LL_Nnew + (1.0d0-revise)*LL_N
      LL_N = max(LL_N,m)
      taul = revise*taulnew + (1.0d0-revise)*taul
      if (nodropout==1) then
        grantprop=revise*grantpropnew + (1.0d0-revise)*grantprop
      ! elseif(heterostddc==1)then
      !   grantprop=revise*grantpropnew + (1.0d0-revise)*grantprop
      else
        grantprop=revise10*grantpropnew + (1.0d0-revise10)*grantprop
      endif

    else

      fval2(1) = K_LLnew-K_LL
      fval2(2) = H_LL1new-H_LL1
      fval2(3) = LL_Nnew-LL_N
      fval2(4) = taulnew-taul



      if (debug==1) then

        write(*,*) 'moments',K_LL, H_LL1, LL_N, taul
      endif

      K_LL = revise*K_LLnew + (1.0d0-revise)*K_LL
      K_LL = max(K_LL,m)
      H_LL1 = revise3*H_LL1new + (1.0d0-revise3)*H_LL1
      H_LL1 = max(H_LL1,m)
      LL_N = revise*LL_Nnew + (1.0d0-revise)*LL_N
      LL_N = max(LL_N,m)
      taul = revise*taulnew + (1.0d0-revise)*taul

      dddddd=maxval(abs(fval2))

    endif

  endif

endif

end subroutine modelsolution
