subroutine param

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!!  Kazushige MATSUDA
  !!!!  "Progressive Taxation vs College Subsidies with College Dropout"
  !!!!
  !!!!
  !!!!  Routine for generating parameters
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  use globals
  use interface, only : normcdf, linspace
  use nr, only : polint, golden

  implicit none

  integer :: iii ,jjj
  real(DP),dimension(101,6)::ageprod
  integer :: jcc, fo=16
  real(DP)::survpop(jage)
  real(DP),dimension(jage+1)::usmr

  ind_fb(:)=0.0d0
  ind_fb(jf:jb-1)=1.0d0

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Survival rates
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  open(99,file='lifetables2000.txt')
  read(99,*) usmr ! prob of dying between ages t and t+1
  close(99)
  survpop(1:jage)=1.0d0-usmr(2:jage+1)    ! survival to next year

  jcc=jaage
  do jj=ja,jend-1
    varphi(jj)=survpop(jcc)*survpop(jcc+1)   ! survival 2 years later
    jcc=jcc+dj
  enddo

  ! restrict survival rates: approximation (simplification): no death until children leave the household
  varphi(1:jr-1)=1.0d0
  varphi(jend)=0.0d0

  Nvec(1) = 1.0d0
  do jj=2,jend,1
    Nvec(jj) = varphi(jj-1)/(1.0d0+cchi)*Nvec(jj-1)
  enddo
  Nvec=Nvec/sum(Nvec(ja:jend))
  NN=1.0d0


  betav=0.0d0
  do jj=ja,jend
    betav=betav+bbeta**(jj-ja)*varphi(jj)
  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Age profile
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  ! pii(s,eeta,eetaprime) = probability from state eeta to eetaprime for education s

  do s=1,snum
    sigmas = sigmaeta(s)/sqrt(1.0d0-rhoo(s)**2)
    eeta(s,1) = exp(sigmas)
    eeta(s,2) = exp(-sigmas)
    pii(s,1,1) = (rhoo(s)**dj+1)*0.5d0
    pii(s,2,2) = pii(s,1,1)
    pii(s,1,2) = 1.0d0-pii(s,1,1)
    pii(s,2,1) = 1.0d0-pii(s,2,2)
    pis(s,1) = 0.5d0
    pis(s,2) = 1.0d0-pis(s,1)
  enddo

  ageprod(:,:)=0.0d0
  do s=1,snum
    do jj=jaage,jrage
      ageprod(jj,s)=exp(expcoeff1(s)*dble(jj)+expcoeff2(s)*dble(jj)**2)
    enddo
  enddo

  jcc=1+jaage
  do jj=ja,jr-1
    do s=1,snum
      aap(s,jj)=sum(ageprod(jcc:jcc+dj-1,s))/dble(dj)
    enddo
    jcc=jcc+dj
  end do

  aap(1,:)=aap(1,:)/aap(1,ja+2)
  aap(2,:)=aap(2,:)/aap(2,ja+1)*a2
  aap(3,:)=aap(3,:)/aap(3,ja)


  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Ability-wage profile
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


  hgrid2(:)=linspace(meanh-0.55d0, meanh+0.55d0,hnum2)

  ! standard deviation of thetac given thetah: stdsigmac(thetah)
  if (heterostddc==1) then
    stdsigmac(:)=stddc*exp(stddc1*(hgrid2(:)-meanh))
    ! do hh=1,hnum2
    !   stdsigmac(hh)=max(stddc+stddc1*(hgrid2(hh)-meanh),m)
    ! enddo
  else
    stdsigmac(:)=stddc
  endif

  ! range of thetac
  hgrid3(:)=linspace(hgrid2(1)-1.7505020085*stdsigmac(1), hgrid2(hnum2)+1.7505020085*stdsigmac(hnum2),hnum2)

  if (hnum2==21 .and. hnum==5) then
    hhm(1:5) = (/1,6,11,16,21 /)
  else
    stop 'error on the number of grids of ability h'
  endif

  do hh=1,hnum
    hgrid(hh)=hgrid2(hhm(hh))
    hgrid4(hh)=hgrid3(hhm(hh))
  enddo

  ghgrid(1,:) = afqtcoeff(1)*(hgrid4(:)-meanh)
  ghgridinterp(1,:) = afqtcoeff(1)*(hgrid3(:)-meanh)
  ghgrid(2,:) = afqtcoeff(2)*(hgrid(:)-meanh)
  ghgridinterp(2,:) = afqtcoeff(2)*(hgrid2(:)-meanh)
  ghgrid(3,:) = afqtcoeff(3)*(hgrid(:)-meanh)
  ghgridinterp(3,:) = afqtcoeff(3)*(hgrid2(:)-meanh)

  ghgrid(:,:)=exp(ghgrid(:,:))
  ghgridinterp(:,:)=exp(ghgridinterp(:,:))

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Inter-generational transmission of ability
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


  mup(1,:)=mupbar+meanh+cchi2*(hgrid3(:)-meanh)
  mup(2,:)=mupbar+meanh+cchi2*(hgrid2(:)-meanh)
  mup(3,:)=mupbar+meanh+cchi2*(hgrid2(:)-meanh)

  bopt(:)=bopt0+bopt1*(hgrid2(:)-meanh)

  ! intergenerational transmission: psiem(hc,s,hh) is the prob of abil hc given parents educ s and abil hh
  do s=1,snum
    do hh=1,hnum2
      do hc=1,hnum2
        if (hc==1) then
          psiem(hc,s,hh) = normcdf(hgrid2(1)+(hgrid2(2)-hgrid2(1))*0.5d0-mup(s,hh),0.0d0,ssigmae)
        else if (hc==hnum2) then
          psiem(hc,s,hh) = 1.0d0-normcdf(hgrid2(hnum2)-(hgrid2(hnum2)-hgrid2(hnum2-1))*0.5d0-mup(s,hh),0.0d0,ssigmae)
        else
          psiem(hc,s,hh) = normcdf(hgrid2(hc)+(hgrid2(hc+1)-hgrid2(hc))*0.5d0-mup(s,hh),0.0d0,ssigmae) &
          -normcdf(hgrid2(hc)-(hgrid2(hc)-hgrid2(hc-1))*0.5d0-mup(s,hh),0.0d0,ssigmae)
        endif
      enddo
      psiem(:,s,hh) = psiem(:,s,hh)/sum(psiem(:,s,hh))
    enddo
  enddo

  ! high school ability to college ability transition
  do iii=1,hnum2
    do jjj=1,hnum2
      if (jjj==1) then
        pepmat(iii,1) = normcdf(hgrid3(1)+(hgrid3(2)-hgrid3(1))*0.5d0-hgrid2(iii),0.0d0,stdsigmac(iii))
      else if (jjj==hnum2) then
        pepmat(iii,hnum2) = 1.0d0-normcdf(hgrid3(hnum2)-(hgrid3(hnum2)-hgrid3(hnum2-1))*0.5d0-hgrid2(iii),0.0d0,stdsigmac(iii))
      else
        pepmat(iii,jjj) = normcdf(hgrid3(jjj) + (hgrid3(jjj+1)-hgrid3(jjj))*0.5d0-hgrid2(iii),0.0d0,stdsigmac(iii)) &
        -normcdf(hgrid3(jjj) - (hgrid3(jjj)-hgrid3(jjj-1))*0.5d0-hgrid2(iii),0.0d0,stdsigmac(iii))
      endif
    enddo
    pepmat(iii,:)=pepmat(iii,:)/sum(pepmat(iii,:))
  enddo

  ! Optimistic high school ability to college ability transition
  do iii=1,hnum2
    do jjj=1,hnum2
      if (jjj==1) then
        pepmatopt(iii,1) = normcdf(hgrid3(1)+(hgrid3(2)-hgrid3(1))*0.5d0-hgrid2(iii)-bopt(iii),0.0d0,stdsigmac(iii))
      else if (jjj==hnum2) then
        pepmatopt(iii,hnum2) = 1.0d0&
        -normcdf(hgrid3(hnum2) - (hgrid3(hnum2)-hgrid3(hnum2-1))*0.5d0-hgrid2(iii)-bopt(iii),0.0d0,stdsigmac(iii))
      else
        pepmatopt(iii,jjj) = normcdf(hgrid3(jjj) + (hgrid3(jjj+1)-hgrid3(jjj))*0.5d0-hgrid2(iii)-bopt(iii),0.0d0,stdsigmac(iii)) &
        -normcdf(hgrid3(jjj) - (hgrid3(jjj)-hgrid3(jjj-1))*0.5d0-hgrid2(iii)-bopt(iii),0.0d0,stdsigmac(iii))
      endif
    enddo
    if (sum(pepmatopt(iii,:)) < m) then
      pepmatopt(iii,hnum)=1.0d0
    else
      pepmatopt(iii,:)=pepmatopt(iii,:)/sum(pepmatopt(iii,:))
    endif
  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! College taste
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


  kegrid0(:)=linspace(-2.0d0,2.0d0,kenum)

  do ke=1,kenum
    if (ke==1) then
      pike(ke) = normcdf(kegrid0(1)+(kegrid0(2)-kegrid0(1))*0.5d0,0.0d0,1.0d0)
    else if (ke==kenum) then
      pike(ke) = 1.0d0-normcdf(kegrid0(kenum)-(kegrid0(kenum)-kegrid0(kenum-1))*0.5d0,0.0d0,1.0d0)
    else
      pike(ke) = normcdf(kegrid0(ke)+(kegrid0(ke+1)-kegrid0(ke))*0.5d0,0.0d0,1.0d0) &
      -normcdf(kegrid0(ke)-(kegrid0(ke)-kegrid0(ke-1))*0.5d0,0.0d0,1.0d0)
    endif
  enddo
  pike(:) = pike(:)/sum(pike(:))

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Utility of college attendance
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  do hh=1,hnum2
    lambdaopt(ja+1,hh,:)=-lambda0-lambda1*(hgrid3(hh)-meanh)+kegrid0(:)*elast2
    lambda(ja+1,hh,:)=-lambda0-lambda1*(hgrid3(hh)-meanh)+kegrid0(:)*elast2

    lambdaopt(ja,hh,:)=-lambda0-lambda1*(hgrid2(hh)+bopt(hh)-meanh)+kegrid0(:)*elast
    lambda(ja,hh,:)=-lambda0-lambda1*(hgrid2(hh)-meanh)+kegrid0(:)*elast
  enddo




end subroutine param
